
def define_patch_workflow(add_workflow):
    pwf = add_workflow(_('patch workflow'), 'Patch')

    inprogress = pwf.add_state(_('in-progress'), initial=True)
    pending    = pwf.add_state(_('pending-review'))
    reviewed   = pwf.add_state(_('reviewed'))
    applied    = pwf.add_state(_('applied'))
    rejected   = pwf.add_state(_('rejected'))
    folded     = pwf.add_state(_('folded'))
    outdated   = pwf.add_state(_('outdated'))

    pwf.add_transition(_('ask review'), (inprogress, reviewed, outdated), pending,
                       ('managers', 'users',)) # XXX patch owner only?
    pwf.add_transition(_('accept'), (inprogress, pending), reviewed,
                       ('managers', 'committers', 'reviewers',),
                       ('X patch_revision RE, RE from_repository R, R repository_committer U',
                        'X patch_revision RE, RE from_repository R, R repository_reviewer U')) # XXX patch_reviewer/repo manager only?
    pwf.add_transition(_('ask rework'), (pending, reviewed), inprogress,
                       ('managers', 'committers', 'reviewers',),
                       ('X patch_revision RE, RE from_repository R, R repository_committer U',
                        'X patch_revision RE, RE from_repository R, R repository_reviewer U')) # XXX patch_reviewer/repo manager only? ignore patch_reviewer if in reviewed state?
    pwf.add_transition(_('fold'), (inprogress, pending, outdated, reviewed), folded,
                       ('managers', 'committers', 'reviewers',),
                       ('X patch_revision RE, RE from_repository R, R repository_committer U',
                        'X patch_revision RE, RE from_repository R, R repository_reviewer U')) # XXX patch owner/repo mananger only?
    pwf.add_transition(_('apply'), (inprogress, pending, outdated, reviewed), applied,
                       ('managers', 'committers',),
                       'X patch_revision RE, RE from_repository R, R repository_committer U')
    pwf.add_transition(_('reject'), (inprogress, pending, outdated, reviewed), rejected,
                       ('managers', 'committers',),
                       'X patch_revision RE, RE from_repository R, R repository_committer U')
    # internal transition, not available through the ui
    # XXX we've to put 'managers' group since transition without groups
    # nor condition are fireable by anyone...
    pwf.add_transition(_('obsolete'), (inprogress, pending, reviewed), outdated,
                       ('managers',))
    # in case hooks following patch state made a mistake, we should be able to
    # reopen the patch
    pwf.add_transition(_('reopen'), outdated, inprogress,
                       ('managers', 'committers',),
                       'X patch_revision RE, RE from_repository R, R repository_committer U')
    pwf.add_transition(_('reopen/accept'), outdated, reviewed, ('managers', ))



# copyright 2011-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-vcreview postcreate script, executed at instance creation time or when
the cube is added to an existing instance.

You could setup site properties or a workflow here for example.
"""

from cubes.vcreview.workflows import define_patch_workflow

define_patch_workflow(add_workflow)

commit()

# change task workflow permission
conditions = [
    # global reviewer / committer
    'Z has_activity X, U in_group G, G name IN ("reviewers", "committers")',
    # repository reviewer / committer
    'P has_activity X, P patch_revision RE, RE from_repository R, EXISTS(R repository_reviewer U) OR EXISTS(R repository_committer U)',
    'IP has_activity X, IP point_of RE, P patch_revision RE, RE from_repository R, EXISTS(R repository_reviewer U) OR EXISTS(R repository_committer U)',
    # patch owner
    'P has_activity X, P owned_by U',
    'IP has_activity X, IP point_of RE, P patch_revision RE, P owned_by U',
    ]
task_wf = get_workflow_for('Task')
task_wf.transition_by_name('start').set_permissions(conditions=conditions, reset=False)
task_wf.transition_by_name('done').set_permissions(conditions=conditions, reset=False)



import mercurial.revset, mercurial.obsolete
#from mercurial.i18n import _

def _is_pruned(repo, node):
    repo = repo.unfiltered()
    succmarkers = repo.obsstore.successors
    toproceed = set([node])
    seen = set(toproceed)
    while toproceed:
        current = toproceed.pop()
        if current not in succmarkers:
            return False
        for m in succmarkers.get(current, ()):
            for s in m[1]:
                if s not in seen:
                    seen.add(s)
                    toproceed.add(s)
    return True

def pruned(repo, subset, x):
#    mercurial.revset.getargs(x, 0, 0, _('unknown_successor takes no argument'))
    mercurial.revset.getargs(x, 0, 0, 'unknown_successor takes no argument')
    return subset.filter(lambda node: _is_pruned(repo, repo[node].node()))

def extsetup(ui):
    if mercurial.obsolete._enabled:
        mercurial.revset.symbols['pruned'] = pruned
